import arcpy
import os
import numpy as np
from arcpy import env
from arcpy.sa import *
import arcpy.cartography as CA
import re

arcpy.CheckOutExtension('Spatial')
arcpy.overwriteOutput = True
env.qualifiedFieldNames = True
arcpy.env.addOutputsToMap = False


# Import Data
Workspace = arcpy.GetParameterAsText(0) # Workspace with series of buffers
WorkspaceAdd = arcpy.GetParameterAsText(1) # New Workspace for intermediate steps
Buf256 = arcpy.GetParameterAsText(2) # Largest buffer shapefile


Bufs = []
for dirpath, dirnames, filenames in arcpy.da.Walk(Workspace, topdown = True, datatype = 'FeatureClass', type = 'Polygon'):
	for filename in filenames:
		Bufs.append(os.path.join(dirpath, filename))

arcpy.analysis.SplitByAttributes(Buf256, WorkspaceAdd, "ID_LM_EMB")

Bays = []
for dirpath, dirnames, filenames in arcpy.da.Walk(WorkspaceAdd, topdown = True, datatype = 'FeatureClass', type = 'Polygon'):
	for filename in filenames:
		Bays.append(os.path.join(dirpath, filename))

#Clip smaller buffers to 256 km buffer
for fc in Bufs:
	for fc2 in Bays:
		bbb = re.split('[_ .]', fc2) # embayment number
		ccc = re.split('[_ .]', fc) # buffer size
		clip_name = 'C_' + bbb[-2] + '_' + ccc[-2]
		clip_output = os.path.join(WorkspaceAdd, clip_name)
		arcpy.analysis.Clip(fc, fc2, clip_output)

Bays2 = []
for dirpath, dirnames, filenames in arcpy.da.Walk(WorkspaceAdd, topdown = True, datatype = 'FeatureClass', type = 'Polygon'):
	for filename in filenames:
		if filename.startswith('C_'):
			Bays2.append(os.path.join(dirpath, filename))

for fc in Bays2:
	bbb = re.split('[_ .]', fc)
	diss_name = 'D_' + bbb[-3] + '_' + bbb[-2]
	diss_output = os.path.join(WorkspaceAdd, diss_name)
	arcpy.management.Dissolve(fc, diss_output)
	
BaysD = []
for dirpath, dirnames, filenames in arcpy.da.Walk(WorkspaceAdd, topdown = True, datatype = 'FeatureClass', type = 'Polygon'):
	for filename in filenames:
		if filename.startswith('D_'):
			BaysD.append(os.path.join(dirpath, filename))

# Check if field already exists
def FieldExist(featureclass, fieldname):
    fieldList = arcpy.ListFields(featureclass, fieldname)
    fieldCount = len(fieldList)
    if (fieldCount == 1):
        return True
    else:
        return False

ListofFieldNeeded = ['AREA_Msq']
ListofFieldNeeded1 = ['Buffer']
ListofFieldNeeded2 = ['Bay']
ListofFieldNeeded3 = ['Erase']


list_buf = []
for fc in BaysD:
	count = arcpy.GetCount_management(fc)
	count = int(count[0])
	for Afield in ListofFieldNeeded:
		if not FieldExist(fc, Afield):	
			arcpy.management.AddField(fc, "AREA_Msq", "FLOAT", "", "", "", "NULLABLE")
	for Afield in ListofFieldNeeded1:
		if not FieldExist(fc, Afield):	
			arcpy.management.AddField(fc, "Buffer", "SHORT", "", "", "", "NULLABLE")
	for Afield in ListofFieldNeeded2:
		if not FieldExist(fc, Afield):	
			arcpy.management.AddField(fc, "Bay", "SHORT", "", "", "", "NULLABLE")
	for Afield in ListofFieldNeeded3:
		if not FieldExist(fc, Afield):	
			arcpy.management.AddField(fc, "Erase", "SHORT", "", "", "", "NULLABLE")
	if count > 0:
		arcpy.management.CalculateGeometryAttributes(fc, [["AREA_Msq", "AREA"]], "", "SQUARE_METERS")
	else:
		cursor =  arcpy.InsertCursor(fc)
		for x in range (0, 1):
			row = cursor.newRow()
			row.setValue("AREA_Msq", 0)
			cursor.insertRow(row)
	ccc = re.split('[_ .]', fc)
	bay = ccc[-3]
	buf = ccc[-2]
	era = 1
	list_buf.append(buf)
	arcpy.management.CalculateField(fc, "Buffer", buf)
	arcpy.management.CalculateField(fc, "Bay", bay)
	arcpy.management.CalculateField(fc, "Erase", era)
	


#find largest area for each buffer within each embayment
a = []; b = []
for item in list_buf:  #create list with buffer sizes
    if item not in a: 
        a.append(item)
a = [eval(i) for i in a] #convert list of strings to list of ints
for i in range(0,9):
    b.append(i) #create indices for 9 buffer sizes

c = [] #extract area values for all buffers/bays
for fc in BaysD:
	SC = arcpy.SearchCursor(fc)
	for row in SC:
		mm = row.getValue("AREA_Msq")
		c.append(mm)
a1 = sorted(a)
li=[]
for i in range(len(a)):
     li.append([a[i],i])
li.sort()
b1 = []
for x in li:
	b1.append(x[1]) #get sort index

for m in range(0, int(len(c)/9)):
	kk1=9*m+0
	kk2=9*m+9
	c1 = c[kk1:kk2]
	BaysD1 = BaysD[kk1:kk2]
	c11 = [c1[i] for i in b1]
	BaysD11 = [BaysD1[i] for i in b1]
	inc = all(i < j for i, j in zip(c11, c11[1:])) #check if list is strictly increasing?

	if inc == False:
		d = [j-i for i,j in zip(c11[:-1], c11[1:])]
		d1 = np.argwhere(np.array(d) < 0)
		d2 = np.argwhere(np.array(d) == 0)
		if len(d2) > 0:
			for n2 in d2:
				rows = arcpy.UpdateCursor(BaysD11[int(n2)+1])
				for row in rows:
					row.setNull("Erase")
					rows.updateRow(row)                
		if len(d1) > 0:
			for n1 in d1:
				rows = arcpy.UpdateCursor(BaysD11[int(n1)+1])
				for row in rows:
					row.setNull("Erase")
					rows.updateRow(row)  

	


	







