import arcpy
import os
import numpy as np
import re
from arcpy import env
from arcpy.sa import *
import arcpy.cartography as CA

arcpy.CheckOutExtension('Spatial')
arcpy.overwriteOutput = True
env.qualifiedFieldNames = True
arcpy.env.addOutputsToMap = False

# Import Data
Change = arcpy.GetParameterAsText(0) # GSWD area change dataset
WorkspaceBuf = arcpy.GetParameterAsText(1) # Define workspace including series of buffers (Output from tool 2)
WorkspaceInterm = arcpy.GetParameterAsText(2) # Define workspace for intermediate steps 
Country = arcpy.GetParameterAsText(3) # Get shapefile with country borders

# Extract important information from buffer shapefiles
BB = []
for dirpath, dirnames, filenames in arcpy.da.Walk(WorkspaceBuf, topdown = True, datatype = 'FeatureClass', type = 'Polygon'):
    for filename in filenames:
        if filename.startswith("D_"):
            BB.append(os.path.join(dirpath, filename))

bay = []; buf = []; era = []
for fc in BB:
	SC = arcpy.SearchCursor(fc)
	for row in SC:
		m1 = row.getValue("Buffer"); buf.append(m1)
		m2 = row.getValue("Bay"); bay.append(m2)
		m3 = row.getValue("Erase"); era.append(m3)

x = []; y = []
for fc in BB:
	arcpy.management.CalculateGeometryAttributes(fc, [["x", "CENTROID_X"], ["y", "CENTROID_Y"]],"", "", "", "DD")
	SC = arcpy.SearchCursor(fc)
	for row in SC:
		m1 = row.getValue("x"); x.append(m1)
		m2 = row.getValue("y"); y.append(m2)

# Clip change data to country
arcpy.management.SelectLayerByAttribute(Country, 'NEW_SELECTION', "ISO = 'KR'", "NON_INVERT")
buf_name = "CountryB"
buf_output = os.path.join(WorkspaceInterm, buf_name)
CountryB = arcpy.analysis.Buffer(Country, buf_output, "100 Kilometers", "FULL", "ROUND", "NONE")
clip_name2 = "Change2"
clip_output2 = os.path.join(WorkspaceInterm, clip_name2)
Change2 = ExtractByMask(Change, CountryB)
Change2.save(clip_output2)

# Clip change data to individual buffers/bays
for fc in BB:
	ccc = re.split(r'[b\W\b]', fc)
	ccc1 = ccc[-2].split('_')
	clip_name = 'A' + '_' + ccc1[-2] + '_' + ccc1[-1]
	clip_output = os.path.join(WorkspaceInterm, clip_name)
	Clip = ExtractByMask(Change2, fc)
	Clip.save(clip_output)
arcpy.env.workspace = WorkspaceInterm
change_rasters = arcpy.ListRasters("A_*")

# Resample rasters into gain/loss/no change
for raster in change_rasters:
	try:
		samp_name = raster + "samp"
		samp_output = os.path.join(WorkspaceInterm, samp_name)	
		reclass = Reclassify(raster, "VALUE", RemapRange([[0, 99, 1], [100, 100, 2], [101, 200, 3], [253, 255, 4]]))
		reclass.save(samp_output)
	except:
		samp_name = raster + "samp"
		samp_output = os.path.join(WorkspaceInterm, samp_name)	
		arcpy.management.CopyRaster(raster, samp_output)

arcpy.env.workspace = WorkspaceInterm
samp_rasters = arcpy.ListRasters("*samp")

list3 = []
field_names = ['VALUE', 'COUNT']
for raster in samp_rasters:
	with arcpy.da.SearchCursor(raster, field_names) as sc:
		m=-1
		prerow=0
		for m, row in enumerate(sc):
			if m==0 and row[0]!=1: list3.extend(np.tile(0,row[0]-1)); prerow=row[0]-1 #append a number of zeros, the number is row[0]-1
			if row[0]!=prerow+1: list3.extend(np.tile(0,row[0]-prerow-1))                             
			prerow=row[0] 
			list3.append(row[1])
		if prerow>0 and prerow<4: list3.extend(np.tile(0,4-prerow))                              
		if m==-1: list3.extend(np.tile(0,4)) # if empty raster, add 4 zeros

list4 = []
for xx in list3:
    yy = xx *30 * 30 / 1000000
    list4.append(yy)

k = 4
del list4[k-1::k]

bay_samp = []; buf_samp = []
for raster in samp_rasters:
	bbb = re.split('[_ s]', raster)
	bay_samp.append(bbb[-3])
	buf_samp.append(bbb[-2])

bay_samp = [eval(i) for i in bay_samp] #convert list of string to list of ints
buf_samp = [eval(i) for i in buf_samp]

u = buf_samp == buf; ay = bay_samp == bay #make sure lists of bays and buffers from area calculations (samp_rasters) and add-on tool (era) are the same

# if lists do not match, get indices of buf positions in buf_samp
ii = []
if u == False:
	buf = buf[0:9]
	buf_samp = buf_samp[0:9]
	for m in buf_samp:
		i = buf.index(m)
		ii.append(i)

header = "Buffer | Erase (Y/N) | Loss (km2) | No Change (km2) | Gain (km2)"

for m in range(0, int(len(era)/9)):
	kk1 = 9*m+0; kk2 = 9*m+9; kk3 = 27*m+0; kk4 = 27*m+27
	era1 = era[kk1:kk2]; era11 = [era1[i] for i in ii] # sort erase values 
	bay1 = bay[9*m+0]
	x1 = x[kk1:kk2]; y1 = y[kk1:kk2]; x11 = [x1[i] for i in ii]; y11 = [y1[i] for i in ii]
	buf1 = [buf[i] for i in ii] # sort buf values to match samp_bufs
	aa = list4[kk3:kk4] 
	text_name = 'AreaChange_Bay' + str(bay1) + '.txt'
	text = os.path.join(WorkspaceBay, text_name)
	with open(text, 'a') as file_writer:
		file_writer.write(header + "\n")
		file_writer.write(str(buf1[0]) + ',' + str(era11[0]) + ',' + str(aa[0]) + ',' + str(aa[1]) + ',' + str(aa[2]) + ',' + str(x11[0]) + ',' + str(y11[0]) + "\n")
		file_writer.write(str(buf1[1]) + ',' + str(era11[1]) + ','+ str(aa[3]) + ',' + str(aa[4]) + ',' + str(aa[5]) + ',' + str(x11[1]) + ',' + str(y11[1]) + "\n")
		file_writer.write(str(buf1[2]) + ',' + str(era11[2]) + ','+ str(aa[6]) + ',' + str(aa[7]) + ',' + str(aa[8]) + ',' + str(x11[2]) + ',' + str(y11[2]) + "\n")
		file_writer.write(str(buf1[3]) + ',' + str(era11[3]) + ','+ str(aa[9]) + ',' + str(aa[10]) + ',' + str(aa[11]) + ',' + str(x11[3]) + ',' + str(y11[3]) + "\n")
		file_writer.write(str(buf1[4]) + ',' + str(era11[4]) + ','+ str(aa[12]) + ',' + str(aa[13]) + ',' + str(aa[14]) + ',' + str(x11[4]) + ',' + str(y11[4]) + "\n")
		file_writer.write(str(buf1[5]) + ',' + str(era11[5]) + ','+ str(aa[15]) + ',' + str(aa[16]) + ',' + str(aa[17]) + ',' + str(x11[5]) + ',' + str(y11[5]) + "\n")
		file_writer.write(str(buf1[6]) + ',' + str(era11[6]) + ','+ str(aa[18]) + ',' + str(aa[19]) + ',' + str(aa[20]) + ',' + str(x11[6]) + ',' + str(y11[6]) + "\n")
		file_writer.write(str(buf1[7]) + ',' + str(era11[7]) + ','+ str(aa[21]) + ',' + str(aa[22]) + ',' + str(aa[23]) + ',' + str(x11[7]) + ',' + str(y11[7]) + "\n")
		file_writer.write(str(buf1[8]) + ',' + str(era11[8]) + ','+ str(aa[24]) + ',' + str(aa[25]) + ',' + str(aa[26]) + ',' + str(x11[8]) + ',' + str(y11[8]) + "\n")
		file_writer.close


		
###############################################################
############### Extract intensity adjusted area change
###############################################################

val = []; cou = []	
l = []; g = []; nc = []
l1 = []; g1 = []; nc1 = [] 				
for ras in change_rasters:
	SC = arcpy.SearchCursor(ras)	
	for row in SC:
		m1 = row.getValue("VALUE"); val.append(m1)
		m2 = row.getValue("COUNT"); cou.append(m2)
	for i in range(0, len(val)):
		if val[i] < 100:	
			lo = cou[i]*30*30*(abs(val[i]-100)/100)/1000000; l.append(lo)
		elif val[i] == 100:
			no = cou[i]*30*30/1000000; nc.append(no)
		elif 100 < val[i] < 201:
			ga = cou[i]*30*30*((val[i]-100)/100)/1000000; g.append(ga)
	l1.append(sum(l)); g1.append(sum(g)); nc1.append(sum(nc))
	val = []; cou = []; l = []; g = []; nc = []


header = "Buffer | Erase (Y/N) | Loss (km2) | No Change (km2) | Gain (km2)  |  X (DD)  |  Y (DD)" 

for m in range(0, int(len(era)/9)):
	kk1 = 9*m+0; kk2 = 9*m+9;
	era1 = era[kk1:kk2]; 
	bay1 = bay[9*m+0]
	x1 = x[kk1:kk2]; y1 = y[kk1:kk2];
	l11 = l1[kk1:kk2]; g11 = g1[kk1:kk2]; nc11 = nc1[kk1:kk2] 
	text_name = 'AdjAreaChange_Bay' + str(bay1) + '.txt'
	text = os.path.join(WorkspaceBay, text_name)
	with open(text, 'a') as file_writer:
		file_writer.write(header + "\n")
		file_writer.write(str(buf[0]) + ',' + str(era1[0]) + ',' + str(l11[0]) + ',' + str(nc11[0]) + ',' + str(g11[0]) + ',' + str(x1[0]) + ',' + str(y1[0]) + "\n")
		file_writer.write(str(buf[1]) + ',' + str(era1[1]) + ','+ str(l11[1]) + ',' + str(nc11[1]) + ',' + str(g11[1]) + ',' + str(x1[1]) + ',' + str(y1[1]) + "\n")
		file_writer.write(str(buf[2]) + ',' + str(era1[2]) + ','+ str(l11[2]) + ',' + str(nc11[2]) + ',' + str(g11[2]) + ',' + str(x1[2]) + ',' + str(y1[2]) + "\n")
		file_writer.write(str(buf[3]) + ',' + str(era1[3]) + ','+ str(l11[3]) + ',' + str(nc11[3]) + ',' + str(g11[3]) + ',' + str(x1[3]) + ',' + str(y1[3]) + "\n")
		file_writer.write(str(buf[4]) + ',' + str(era1[4]) + ','+ str(l11[4]) + ',' + str(nc11[4]) + ',' + str(g11[4]) + ',' + str(x1[4]) + ',' + str(y1[4]) + "\n")
		file_writer.write(str(buf[5]) + ',' + str(era1[5]) + ','+ str(l11[5]) + ',' + str(nc11[5]) + ',' + str(g11[5]) + ',' + str(x1[5]) + ',' + str(y1[5]) + "\n")
		file_writer.write(str(buf[6]) + ',' + str(era1[6]) + ','+ str(l11[6]) + ',' + str(nc11[6]) + ',' + str(g11[6]) + ',' + str(x1[6]) + ',' + str(y1[6]) + "\n")
		file_writer.write(str(buf[7]) + ',' + str(era1[7]) + ','+ str(l11[7]) + ',' + str(nc11[7]) + ',' + str(g11[7]) + ',' + str(x1[7]) + ',' + str(y1[7]) + "\n")
		file_writer.write(str(buf[8]) + ',' + str(era1[8]) + ','+ str(l11[8]) + ',' + str(nc11[8]) + ',' + str(g11[8]) + ',' + str(x1[8]) + ',' + str(y1[8]) + "\n")
		file_writer.close





















